"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("./constants");

var _getActionsFromSpec = _interopRequireDefault(require("./getActionsFromSpec"));

var _getLocationFromSpec = _interopRequireDefault(require("./getLocationFromSpec"));

var _getSpecFromLocation = _interopRequireDefault(require("./getSpecFromLocation"));

var _getSpecFromState = _interopRequireDefault(require("./getSpecFromState"));

var messages = _interopRequireWildcard(require("./messages"));

var _utils = require("./utils");

var _default = function _default(routes, history) {
  var blockLocationChange = false;

  var storeListener = function storeListener(store) {
    return function () {
      var method = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _constants.PUSH;
      var overrideSpec = arguments.length > 1 ? arguments[1] : undefined;

      if (!blockLocationChange) {
        var state = store.getState();
        var spec = overrideSpec || (0, _getSpecFromState.default)(routes, state);

        if (spec !== undefined) {
          var oldLocation = history.location;
          var newLocation = (0, _getLocationFromSpec.default)(routes, oldLocation, spec);

          if (method === _constants.NEW_TAB) {
            var pathname = newLocation.pathname,
                search = newLocation.search,
                hash = newLocation.hash;
            var url = "".concat(pathname).concat(search).concat(hash);
            window.open(url, '_blank');
          } else if (!(0, _utils.locationsEqual)(oldLocation, newLocation)) {
            switch (method) {
              case _constants.PUSH:
                history.push(newLocation);
                break;

              case _constants.REPLACE:
                history.replace(newLocation);
                break;
            }
          }
        } else {
          console.warn(messages.NO_MATCH_STATE);
        }
      }
    };
  };

  var locationListener = function locationListener(store) {
    return function (location) {
      var spec = (0, _getSpecFromLocation.default)(routes, location);

      if (spec !== undefined) {
        var state = store.getState();
        var actions = (0, _getActionsFromSpec.default)(routes, state, spec);
        blockLocationChange = true;
        actions.forEach(store.dispatch);
        blockLocationChange = false;
        storeListener(store)(_constants.REPLACE);
      } else {
        console.warn(messages.NO_MATCH_LOCATION);
      }
    };
  };

  var middleware = function middleware(store) {
    return function (next) {
      return function (action) {
        if (action.type === _constants.NAVIGATE) {
          var route = (0, _utils.getRoute)(routes, action.payload.route);
          var params = action.payload.params;
          var method = action.meta.method;

          if (route !== undefined) {
            var spec = (0, _utils.createSpec)(route, params);

            if (method === _constants.NEW_TAB) {
              storeListener(store)(method, spec);
            } else {
              var state = store.getState();
              var actions = (0, _getActionsFromSpec.default)(routes, state, spec);
              blockLocationChange = true;
              actions.forEach(store.dispatch);
              blockLocationChange = false;
              storeListener(store)(method);
            }
          } else {
            console.warn(messages.NO_MATCH_ACTION);
          }
        } else {
          next(action);
        }
      };
    };
  };

  var enhancer = function enhancer(createStore) {
    return function () {
      var store = createStore.apply(void 0, arguments);
      store.subscribe(storeListener(store));
      history.listen(locationListener(store));
      return store;
    };
  };

  var syncLocationWithStore = function syncLocationWithStore(store) {
    storeListener(store)(_constants.REPLACE);
  };

  var syncStoreWithLocation = function syncStoreWithLocation(store) {
    locationListener(store)(history.location);
  };

  return {
    enhancer: enhancer,
    middleware: middleware,
    syncLocationWithStore: syncLocationWithStore,
    syncStoreWithLocation: syncStoreWithLocation
  };
};

exports.default = _default;